"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.userspaceContact = exports.makeUserspaceLayer = exports.makeBundledWgQuickLayer = exports.WireguardControl = exports.UserspaceLayer = exports.BundledWgQuickLayer = void 0;
var NodeSocket = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform-node/NodeSocket"));
var Command = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/Command"));
var CommandExecutor = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/CommandExecutor"));
var FileSystem = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/FileSystem"));
var Path = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/Path"));
var Schema = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/schema/Schema"));
var Chunk = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Chunk"));
var Context = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Context"));
var Effect = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Effect"));
var Function = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Function"));
var Layer = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Layer"));
var Option = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Option"));
var Sink = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Sink"));
var Stream = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Stream"));
var String = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/String"));
var exec = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("node:child_process"));
var WireguardConfig = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardConfig.js"));
var WireguardErrors = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardErrors.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/**
 * Wireguard control mechanisms.
 *
 * @since 1.0.0
 */

/**
 * @since 1.0.0
 * @category Tags
 */
const WireguardControl = exports.WireguardControl = /*#__PURE__*/Context.GenericTag("@leonitousconforti/the-wireguard-effect/WireguardControl");
/** @internal */
const userspaceContact = (wireguardInterface, content) => Function.pipe(Stream.make(content), Stream.pipeThroughChannelOrFail(NodeSocket.makeNetChannel({
  path: wireguardInterface.SocketLocation
})), Stream.decodeText(), Stream.flatMap(Function.compose(String.linesIterator, Stream.fromIterable)), Stream.map(String.trimEnd), Stream.filter(String.isNonEmpty), Stream.run(Sink.collectAll()), Effect.tap(Function.flow(Chunk.last, Option.getOrThrow, Schema.decodeUnknown(WireguardErrors.SuccessErrno))), Effect.map(Chunk.join("\n")));
/**
 * @since 1.0.0
 * @category Constructors
 */
exports.userspaceContact = userspaceContact;
const makeUserspaceLayer = () => {
  const up = (wireguardConfig, wireguardInterface) => Effect.request(new WireguardConfig.WireguardSetConfigRequest({
    config: wireguardConfig,
    wireguardInterface
  }), WireguardConfig.WireguardSetConfigResolver);
  const down = (_wireguardConfig, wireguardInterface) => Effect.flatMap(FileSystem.FileSystem, fs => fs.remove(wireguardInterface.SocketLocation)).pipe(Effect.map(Function.constant(wireguardInterface)));
  const upScoped = (wireguardConfig, wireguardInterface) => Effect.acquireRelease(up(wireguardConfig, wireguardInterface), () => down(wireguardConfig, wireguardInterface).pipe(Effect.orDie));
  return WireguardControl.of({
    up,
    down,
    upScoped,
    getConfigRequestResolver: WireguardConfig.WireguardGetConfigResolver,
    setConfigRequestResolver: WireguardConfig.WireguardSetConfigResolver
  });
};
/**
 * @since 1.0.0
 * @category Constructors
 */
exports.makeUserspaceLayer = makeUserspaceLayer;
const makeBundledWgQuickLayer = options => {
  const execCommand = (command, ...args) => process.platform === "win32" && command.includes("-wireguard-go.exe") ? Effect.tryPromise(() => {
    const subprocess = exec.spawn(command, args, {
      detached: true,
      stdio: "ignore"
    });
    subprocess.unref();
    return new Promise(resolve => setTimeout(resolve, 10000));
  }) : Effect.flatMap(CommandExecutor.CommandExecutor, executor => executor.exitCode(options.sudo && process.platform !== "win32" ? Command.make("sudo", command, ...args) : Command.make(`${command}`, ...args)));
  const up = (wireguardConfig, wireguardInterface) => Effect.gen(function* () {
    const path = yield* Path.Path;
    const fs = yield* FileSystem.FileSystem;
    const tempDirectory = yield* fs.makeTempDirectory();
    const file = path.join(tempDirectory, `${wireguardInterface.Name}.conf`);
    yield* wireguardConfig.writeToFile(file);
    const arch = process.arch === "x64" ? "amd64" : process.arch;
    const extension = process.platform === "win32" ? ".exe" : "";
    const wireguardGoUrl = new URL(`./${process.platform}-${arch}-wireguard-go${extension}`, import.meta.url);
    const bundledWireguardGoExecutablePath = yield* path.fromFileUrl(wireguardGoUrl);
    yield* fs.access(bundledWireguardGoExecutablePath, {
      ok: true
    });
    const wgQuickUrl = new URL(`./${process.platform}-wg-quick`, import.meta.url);
    const bundledWgQuickExecutablePath = yield* path.fromFileUrl(wgQuickUrl);
    if (process.platform !== "win32") yield* fs.access(bundledWgQuickExecutablePath, {
      ok: true
    });
    const wgWindowsUrl = new URL(`./win32-${arch}-wireguard.exe`, import.meta.url);
    const bundledWgWindowsExecutablePath = yield* path.fromFileUrl(wgWindowsUrl);
    if (process.platform === "win32") yield* fs.access(bundledWgWindowsExecutablePath, {
      ok: true
    });
    const wgQuickCommandNix = [bundledWgQuickExecutablePath, "up", file];
    const wgQuickCommandWin = [bundledWgWindowsExecutablePath, "/installtunnelservice", file];
    const wgQuickCommand = process.platform === "win32" ? wgQuickCommandWin : wgQuickCommandNix;
    yield* execCommand(bundledWireguardGoExecutablePath, wireguardInterface.Name);
    yield* Effect.request(new WireguardConfig.WireguardSetConfigRequest({
      config: wireguardConfig,
      wireguardInterface
    }), WireguardConfig.WireguardSetConfigResolver);
    yield* execCommand(wgQuickCommand[0], ...wgQuickCommand.slice(1));
    return wireguardInterface;
  });
  const down = (wireguardConfig, wireguardInterface) => Effect.gen(function* () {
    const path = yield* Path.Path;
    const fs = yield* FileSystem.FileSystem;
    const tempDirectory = yield* fs.makeTempDirectory();
    const file = path.join(tempDirectory, `${wireguardInterface.Name}.conf`);
    yield* wireguardConfig.writeToFile(file);
    const arch = process.arch === "x64" ? "amd64" : process.arch;
    const wgQuickUrl = new URL(`./${process.platform}-wg-quick`, import.meta.url);
    const bundledWgQuickExecutablePath = yield* path.fromFileUrl(wgQuickUrl);
    if (process.platform !== "win32") yield* fs.access(bundledWgQuickExecutablePath, {
      ok: true
    });
    const wgWindowsUrl = new URL(`./win32-${arch}-wireguard.exe`, import.meta.url);
    const bundledWgWindowsExecutablePath = yield* path.fromFileUrl(wgWindowsUrl);
    if (process.platform === "win32") yield* fs.access(bundledWgWindowsExecutablePath, {
      ok: true
    });
    const wgQuickCommandWin = [bundledWgWindowsExecutablePath, "/uninstalltunnelservice", wireguardInterface.Name];
    const wgQuickCommandNix = [bundledWgQuickExecutablePath, "down", file];
    const wgQuickCommand = process.platform === "win32" ? wgQuickCommandWin : wgQuickCommandNix;
    yield* execCommand(wgQuickCommand[0], ...wgQuickCommand.slice(1));
    return wireguardInterface;
  });
  const upScoped = (wireguardConfig, wireguardInterface) => Effect.acquireRelease(up(wireguardConfig, wireguardInterface), () => down(wireguardConfig, wireguardInterface).pipe(Effect.orDie));
  return WireguardControl.of({
    up,
    down,
    upScoped,
    getConfigRequestResolver: WireguardConfig.WireguardGetConfigResolver,
    setConfigRequestResolver: WireguardConfig.WireguardSetConfigResolver
  });
};
/**
 * @since 1.0.0
 * @category Layers
 */
exports.makeBundledWgQuickLayer = makeBundledWgQuickLayer;
const UserspaceLayer = exports.UserspaceLayer = /*#__PURE__*/Layer.sync(WireguardControl, makeUserspaceLayer);
/**
 * @since 1.0.0
 * @category Layers
 */
const BundledWgQuickLayer = exports.BundledWgQuickLayer = /*#__PURE__*/Layer.sync(WireguardControl, () => makeBundledWgQuickLayer({
  sudo: true
}));
//# sourceMappingURL=WireguardControl.js.map